/**
 * Assignment #8: POSIX File System Functions
 * Sample Solution
 *
 * Instructor: Ron Mak
 * CMPE 142 Operating Systems
 * Computer Engineering Department
 * San Jose State University
 */
#ifndef WATCHER_H_
#define WATCHER_H_

#include <iostream>
#include <fcntl.h>
#include <unistd.h>

using namespace std;

class Watcher
{
private:
    static const int BUFFER_SIZE = 8192;
    static const int READ_TRIES  = 3;

    int read_fid;
    char buffer[BUFFER_SIZE];

public:
    Watcher(int id) : read_fid(id)
    {
        fcntl(read_fid, F_SETFL, O_NONBLOCK);
    }

    /**
     * Overload the () operators to make a function object (functor).
     */
    void operator() (string command, void command_function())
    {
        cout << "----- " << command << endl;
        command_function();

        int nbytes;
        for (int i = 0; (i < READ_TRIES) && ((nbytes = read_fswatch()) <= 0); i++);

        if (nbytes > 0)
        {
            buffer[nbytes] = '\0';
            cout << buffer;
        }
    }

    int read_fswatch()
    {
        bool eof = false;
        bool got_output = false;

        ssize_t nbytes = read(read_fid, buffer, sizeof(buffer));

        switch (nbytes)
        {
            case -1:
            {
                if (errno == EAGAIN)
                {
                    // Pipe empty. Snooze for another try.
                    sleep(1);
                    break;
                }
                else exit(-2);
            }

            case 0:
            {
                // End of file.
                eof = true;
                break;
            }

            default:
            {
                // Successful read.
                got_output = true;
                break;
            }
        }

        return nbytes;
    }
};

#endif /* WATCHER_H_ */
